require "./body"
require "../vector"

module Gravity
  # A `Gravity::Field` contains several `Gravity::Body` that warp space-time.
  # It accelerates any point given to it using `#acceleration`
  # A gravity field exists in N dimensions.
  # All vectors and body must also be N dimensional.
  class Field(N)
    getter bodies : Array(Body(N))

    # standard constructor
    def initialize(@bodies : Array(Body(N)) = Array(Body(N)).new)
    end

    # standard constructor
    def initialize(bodies : Tuple(Body(N)))
      @bodies = bodies.to_a
    end

    def self.zero
      new(bodies: Array(Body(N)).new)
    end

    # helping constructor for testing and sandboxes
    # - bodies must be a set of `Gravity::Body`
    def initialize(*bodies)
      @bodies = bodies.to_a
    end

    # Generate an acceleration vector based on all bodies of the gravity field
    def acceleration(position : Vector(T, N)) : Vector(T, N) forall T
      acceleration_vector = position.class.zero
      # acceleration_vector = Vector[0.0, 0.0]
      @bodies.reduce(acceleration_vector) do |base, body|
        add_vector = body.acceleration(position)
        base.add!(add_vector)
        base
      end
      acceleration_vector
    end
  end
end
