require "crsfml"

require "./gravity"

class Projectable(N)
  property :gravity_body, :color, :size, :outline_color, :outline_size

  @@hook_new : Proc(Projectable(2), Nil) = ->(obj : Projectable(2)) { nil }
  def self.set_hook_new(hook_proc : Proc(Projectable(2), Nil))
    @@hook_new = hook_proc
  end

  def trigger_hook_new
    @@hook_new.call(self)
  end

  def initialize(
    @gravity_body : Gravity::Body(N),
    @color : Tuple(Int32, Int32, Int32),
    @size : Int32,
    @outline_color : Tuple(Int32, Int32, Int32) = {0, 0, 0},
    @outline_size : Int32 = 0
     )
    trigger_hook_new
  end

  def name
    @gravity_body.name
  end

  def position
    @gravity_body.position
  end

  def acceleration
    @gravity_body.acceleration
  end

  def speed
    @gravity_body.speed
  end

  def sf_shape(offset : Vector(Float64, N) = Vector(Float64, N).zero)
    shape = SF::CircleShape.new(@size)
    shape.fill_color = SF.color(@color[0], @color[1], @color[2])

    shape.outline_thickness = @outline_size
    shape.outline_color = SF.color(@outline_color[0], @outline_color[1], @outline_color[2])
    shape.position = sf_position(offset)
    shape
  end

  # the sf_position is the projected position of the gravity_body on the graphic plan
  def sf_position(offset : Vector(Float64, N) = Vector(Float64, N).zero)
    zoomed_position = @gravity_body.position + offset
    SF.vector2(zoomed_position[0], zoomed_position[1])
  end
end
