# # this is a sloppy monkey patch that only works because I don't manipulate types in a complex way
# struct Int
#   def clear_div(right : Number)
#     self // right
#   end
# end
# struct Float
#   def clear_div(right : Number)
#     self / right
#   end
# end

# alias Vector = Slice
class Vector(T, N) # should be a struct since it's a simple pointer inside, the rest is compile time
  @buffer : Slice(T)
  include Indexable::Mutable(T)

  def initialize(@buffer)
  end

  def self.new(&block : Int32 -> T)
    pointer = Pointer(T).malloc(N, &block)
    slice = Slice.new(pointer, N)
    Vector(T, N).new(slice)
  end

  def self.zero
    new { T.zero }
  end

  macro [](*args)
    %args_type = typeof({{*args}})
    %args_type_size = sizeof(typeof({{*args}}))
    %args_size = {{args.size}}
    %pointer = Pointer(typeof({{*args}})).malloc(%args_size) do |i|
      ({{args}})[i]
    end
    %slice = Slice.new(%pointer, %args_size)
    Vector(typeof({{*args}}), {{args.size}}).new(%slice)
  end

  def pretty_print(pp)
    pp.list("Vector[", @buffer, "]")
  end

  def to_s(io : IO) : Nil
    io << "Vector["
    join io, ", ", &.inspect(io)
    io << ']'
  end

  def size
    N
  end

  @[AlwaysInline]
  def unsafe_fetch(index : Int) : T
    @buffer[index]
  end

  @[AlwaysInline]
  def unsafe_put(index : Int, value : T)
    @buffer[index] = value
  end

  def !=(right : Vector(U, N)) forall U
    size.times do |index|
      return true if to_unsafe[index] != right.to_unsafe[index]
    end

    return false
  end

  def ==(right : Vector(U, N)) forall U
    !(self != right)
  end

  def add!(right : Vector(U, N)) forall U
    size.times do |index|
      @buffer[index] += right[index]
    end
    self
  end

  def clone
    # note: can be optimized using Slice#copy
    Vector(T, N).new { |index| self[index] }
  end

  def +(right : Vector(U, N)) forall U
    clone = self.clone
    size.times do |index|
      clone.to_unsafe[index] += right[index]
    end
    clone
  end

  def -
    clone = self.clone
    size.times do |index|
      clone.to_unsafe[index] = -clone.to_unsafe[index]
    end
    clone
  end

  def -(right : Vector(U, N)) forall U
    clone = self.clone
    size.times do |index|
      clone.to_unsafe[index] -= right[index]
    end
    clone
  end

  def mult!(right : Number)
    size.times do |index|
      @buffer[index] *= right
    end
    self
  end

  def div!(right : Number)
    size.times do |index|
      klass = @buffer[index].class
      @buffer[index] =
        klass.new(
          if klass < Float
            @buffer[index] / right
          elsif klass < Int
            @buffer[index] // right
          else
            @buffer[index] / right
          end
        )
    end
    self
  end

  def *(right : Number)
    clone = self.clone
    clone.mult!(right)
    clone
  end

  def /(right : Number)
    clone = self.clone
    clone.div!(right)
    clone
  end

  def %(right)
    clone = self.clone
    size.times do |index|
      clone.to_unsafe[index] %= right
    end
    clone
  end

  def zero!
    N.times do |index|
      self[index] = T.zero
    end
  end

  def to_unsafe
    @buffer
  end

  def tuple_type
    T
  end

  def magnitude : Number
    reduce(T.zero) { |base, elem| base + elem**2 } ** (1.0 / 2.0)
  end

  # put the vector on a trigonomic circle so sqrt(x**2 + y**2) = 1
  def normalize
    self / magnitude
  end

  def normalize
    self.div!(magnitude)
  end
end
