module PhysicsSandboxDraw

  include ImGui::Helper

  # will yield the value
  def draw_slider(label : String, data : Float64, max : Float64 = 100.0, min : Float64 = -100.0, flags : ImGui::ImGuiSliderFlags = ImGui::ImGuiSliderFlags::NoRoundToFormat, &)
    ptr = pointerof(data)
    if ImGui.slider_scalar(
         label: label,
         p_data: ptr,
         p_min: min,
         p_max: max,
         flags: flags,
       )
      yield data
    end
  end

  # will yield the value
  def draw_input(label : String, data : Float64, max : Float64 = 100.0, min : Float64 = -100.0, flags : ImGui::ImGuiInputTextFlags = ImGui::ImGuiInputTextFlags::EnterReturnsTrue, &)
    ptr = pointerof(data)
    if ImGui.input_scalar(
         label: label,
         p_data: ptr,
         flags: flags,
       )
      data = max if data > max
      data = min if data < min
      yield data
    end
  end

  def draw_captain
    ImGui.text "Time passed: #{@time_passed}"
    draw_slider(
      label: "Set timescale###set_timescale",
      data: @timescale,
      min: 0.1,
      max: 60,
      # max: 3600 * 24 * 7,
      flags: (
        ImGui::ImGuiSliderFlags::Logarithmic
      ),
    ) do |value|
      @timescale = value
    end

  end

  def draw_pilot
    draw_table(title: "Trusters", headers: {"Name", "X", "Y", "unit"}) do
      draw_table_line(
        "Analogic acceleration",
        ->{
          draw_slider(
            label: "###acceleration_analogue_x",
            data: @vessel.gravity_body.acceleration[0],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
            flags: (
              ImGui::ImGuiSliderFlags::Logarithmic
            ),
          ) do |value|
            @vessel.pilot.order(priority: 10, group: :acceleration_x) do
              @vessel.pilot.set_exact_acceleration_axis(0, value)
              @vessel.pilot.permanent_modes.clear
              @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::Constant
              @vessel.pilot.low_thrust_pressed = false
              @vessel.pilot.low_thrust_save = value
            end
          end
        },
        ->{
          draw_slider(
            label: "###acceleration_analogue_y",
            data: @vessel.gravity_body.acceleration[1],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
            flags: (
              ImGui::ImGuiSliderFlags::Logarithmic
            ),
          ) do |value|
            @vessel.pilot.order(priority: 10, group: :acceleration_y) do
              @vessel.pilot.set_exact_acceleration_axis(1, value)
              @vessel.pilot.permanent_modes.clear
              @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::Constant
              @vessel.pilot.low_thrust_pressed = false
              @vessel.pilot.low_thrust_save = value
            end
          end
        },
        "m/s²",
      )

      # automatically reset acceleration to 0, used for maneuvers
      draw_table_line(
        "Low thrust",
        ->{
          draw_slider(
            label: "###acceleration_low_analogic_x",
            data: @vessel.gravity_body.acceleration[0],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
          ) do |value|
            @vessel.pilot.order(priority: 10, group: :maneuvers) do
              @vessel.pilot.temporary_modes << Vessel::Pilot::Mode::Pulse
              if !@vessel.pilot.low_thrust_pressed
                @vessel.pilot.low_thrust_save = @vessel.gravity_body.acceleration[0]
              end
              @vessel.pilot.low_thrust_pressed = true
              @vessel.pilot.set_exact_acceleration_axis(0, value)
              lock_callback = @ui_events_handler.add(SF::Event::KeyPressed) do |callback, event|
                if event.as(SF::Event::KeyPressed).code == SF::Keyboard::Key::L
                  @vessel.pilot.low_thrust_save = value
                end
              end
              @ui_events_handler.add(SF::Event::MouseButtonReleased) do |callback|
                @ui_events_handler.remove(SF::Event::MouseButtonReleased, callback)
                @ui_events_handler.remove(SF::Event::MouseButtonReleased, lock_callback)
                @vessel.pilot.low_thrust_pressed = false
                @vessel.pilot.set_exact_acceleration_axis(0, @vessel.pilot.low_thrust_save)
              end
            end
          end
        },
        ->{
          draw_slider(
            label: "###acceleration_low_analogic_y",
            data: @vessel.gravity_body.acceleration[1],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
          ) do |value|
            @vessel.pilot.order(priority: 10, group: :maneuvers) do
              @vessel.pilot.temporary_modes << Vessel::Pilot::Mode::Pulse
              if !@vessel.pilot.low_thrust_pressed
                @vessel.pilot.low_thrust_save = @vessel.gravity_body.acceleration[1]
              end
              @vessel.pilot.low_thrust_pressed = true
              @vessel.pilot.set_exact_acceleration_axis(1, value)
              lock_callback = @ui_events_handler.add(SF::Event::KeyPressed) do |callback, event|
                if event.as(SF::Event::KeyPressed).code == SF::Keyboard::Key::L
                  @vessel.pilot.low_thrust_save = value
                end
              end
              @ui_events_handler.add(SF::Event::MouseButtonReleased) do |callback|
                @ui_events_handler.remove(SF::Event::MouseButtonReleased, callback)
                @ui_events_handler.remove(SF::Event::MouseButtonReleased, lock_callback)
                @vessel.pilot.low_thrust_pressed = false
                @vessel.pilot.set_exact_acceleration_axis(1, @vessel.pilot.low_thrust_save)
              end
            end
          end
        },
        "m/s²",
      )

      draw_table_line(
        "Computer acceleration",
        ->{
          draw_input(
            label: "###acceleration_digit_x",
            data: @vessel.pilot.computer_input_thrust[0],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
          ) do |value|
            @vessel.pilot.order(priority: 9, group: :maneuvers) do
              @vessel.pilot.computer_input_thrust[0] = value
            end
          end
        },
        ->{
          draw_input(
            label: "###acceleration_digit_y",
            data: @vessel.pilot.computer_input_thrust[1],
            min: -@vessel.pilot.max_thrust,
            max: @vessel.pilot.max_thrust,
          ) do |value|
            @vessel.pilot.order(priority: 9, group: :maneuvers) do
              @vessel.pilot.computer_input_thrust[1] = value
            end
          end
        },
        "m/s²",
      )

      draw_table_line(
        "[Security] Max thurst",
        ->{
          draw_input(
            label: "###pilot_input_max_thurst",
            data: @vessel.pilot.max_thrust,
            min: 9,
            max: 900,
          ) do |value|
            @vessel.pilot.max_thrust = value
          end
        },
        ->{
          draw_slider(
            label: "###pilot_slider_max_thurst",
            data: @vessel.pilot.max_thrust,
            min: 9,
            max: 900,
          ) do |value|
            @vessel.pilot.max_thrust = value
          end
        },
        ->{
          ImGui.text @vessel.pilot.max_thrust > 20 ? "Inertia Dampers needed" : "No danger"
        }
      )

      # Manual reset
      draw_table_line(
        "",
        ->{
          if ImGui.button("Comfirm")
            @vessel.pilot.order(priority: 100, group: :maneuvers) do
              @vessel.pilot.permanent_modes.clear
              @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::Constant
              @vessel.pilot.set_exact_acceleration_axis(@vessel.pilot.computer_input_thrust)
              # @vessel.pilot.set_exact_acceleration_axis(0, @vessel.pilot.computer_input_thrust[0])
              # @vessel.pilot.set_exact_acceleration_axis(1, @vessel.pilot.computer_input_thrust[1])
            end
          end
        },
        ->{
          if ImGui.button("Reset")
            @vessel.pilot.order(priority: 100, group: :prepare) do
              @vessel.pilot.permanent_modes.clear
              @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::Constant
              @vessel.pilot.set_exact_acceleration_axis(@vessel.pilot.computer_input_thrust)
              # @vessel.pilot.computer_input_thrust[0] = @vessel.gravity_body.acceleration[0]
              # @vessel.pilot.computer_input_thrust[1] = @vessel.gravity_body.acceleration[1]
            end
          end
        },
        ->{
          if ImGui.button("Zero")
            @vessel.pilot.order(priority: 10, group: :prepare) do
              @vessel.pilot.computer_input_thrust[0] = 0.0
              @vessel.pilot.computer_input_thrust[1] = 0.0
            end
          end
        },
      )

      draw_table_line(
        "",
        # AntiGrav
        ->{
          if !@vessel.pilot.permanent_modes.includes?(Vessel::Pilot::Mode::AntiGrav)
            if ImGui.button("Anti-grav")
              @vessel.pilot.order(priority: 10, group: :fly_mode) do
                @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::AntiGrav
                # @vessel.gravity_body.acceleration[0] = @vessel.gravity[0]
                # @vessel.gravity_body.acceleration[1] = @vessel.gravity[1]
              end
            end
          else
            if ImGui.button("Disable anti-grav")
              @vessel.pilot.order(priority: 11, group: :fly_mode) do
                if @vessel.pilot.permanent_modes.includes?(Vessel::Pilot::Mode::AntiGrav)
                  @vessel.pilot.permanent_modes.delete Vessel::Pilot::Mode::AntiGrav
                end
              end
            end
          end
        },

        # AntiSpeed
        ->{
          if !@vessel.pilot.permanent_modes.includes?(Vessel::Pilot::Mode::AntiSpeed)
            if ImGui.button("Anti-speed")
              @vessel.pilot.order(priority: 10, group: :fly_mode) do
                @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::AntiSpeed
                @vessel.pilot.current_selected_body = @star.gravity_body
                # @vessel.antispeed[0] = @vessel.speed[0]
                # @vessel.antispeed[1] = @vessel.speed[1]
              end
            end
          else
            if ImGui.button("Disable anti-speed")
              @vessel.pilot.order(priority: 11, group: :fly_mode) do
                if @vessel.pilot.permanent_modes << Vessel::Pilot::Mode::AntiSpeed
                  @vessel.pilot.permanent_modes.delete Vessel::Pilot::Mode::AntiSpeed
                end
              end
            end
          end
        },
      )

    end
  end

  def draw_navigation(gravity_field)
    # ImGui.text "Total Accele is  : [#{acceleration[0].round(4)}, #{acceleration[1].round(4)}] m/s²"
    draw_table(title: "Applied forces", headers: {"Name", "X", "Y", "unit"}) do
      draw_table_line "Pulse Accele", "#{@vessel.gravity_body.acceleration[0].round(4)}", "#{@vessel.gravity_body.acceleration[1].round(4)}", "m/s²"
      draw_table_line "Gravitic Field", "#{gravity_field[0].round(4)}", "#{gravity_field[1].round(4)}", "m/s²"
    end
    ImGui.text "Current speed #{@vessel.gravity_body.speed.magnitude.round(4)} m/s"
    ImGui.text ""

    relative_speed = @vessel.gravity_body.speed - @selected_projectable.speed
    draw_table(title: "Actual vectors", headers: {"Name", "X", "Y", "unit"}) do
      draw_table_line "Position", @vessel.gravity_body.position[0].round(3).to_s, @vessel.gravity_body.position[1].round(3).to_s, "m"
      draw_table_line "Speed", @vessel.gravity_body.speed[0].round(3).to_s, @vessel.gravity_body.speed[1].round(3).to_s, "m/s"
      draw_table_line "Acceleration", @vessel.gravity_body.acceleration[0].round(3).to_s, @vessel.gravity_body.acceleration[1].round(3).to_s, "m/s²"
      draw_table_line "Selected body", "#{@selected_projectable.position[0].round(4)}", "#{@selected_projectable.position[1].round(4)}", "m"
      draw_table_line "", @selected_projectable.name, "", ""
      draw_table_line "Relative speed", "#{relative_speed[0].round(4)}", "#{relative_speed[1].round(4)}", "m/s"
    end
    ImGui.text ""

    ImGui.text "Zoom on"
    ImGui.radio_button("vessel##navigation_zoom", pointerof(@vessel.navigation.zoom_select), 0)
    ImGui.same_line
    ImGui.radio_button("star##navigation_zoom", pointerof(@vessel.navigation.zoom_select), 1)
    ImGui.same_line
    ImGui.radio_button("fixed##navigation_zoom", pointerof(@vessel.navigation.zoom_select), 2)
    ImGui.same_line
    ImGui.radio_button("zero##navigation_zoom", pointerof(@vessel.navigation.zoom_select), 3)
    draw_slider(
      label: "###cadran_slide_zoom_ratio",
      data: @vessel.navigation.zoom_ratio,
      max: 10,
      min: 0.001,
      flags: ImGui::ImGuiSliderFlags::Logarithmic,
    ) do |value|
      @vessel.navigation.zoom_ratio = value
    end

    ImGui.text "Cadran"
    draw_slider(
      label: "###cadran_slide_radius",
      data: @vessel.navigation.cadran.radius[0],
      max: 10.0**10,
      min: 0.0,
      flags: ImGui::ImGuiSliderFlags::Logarithmic,
    ) do |value|
      @vessel.navigation.cadran.radius[0] = value
    end
  end

  def draw_bridge(gravity_field)
    if ImGui.begin("Bridge")
      if ImGui.tree_node_ex("Captain", ImGui::ImGuiTreeNodeFlags.new(ImGui::ImGuiTreeNodeFlags::DefaultOpen))
        draw_captain
        ImGui.tree_pop
      end

      if ImGui.tree_node_ex("Navigation", ImGui::ImGuiTreeNodeFlags.new(ImGui::ImGuiTreeNodeFlags::DefaultOpen))
        draw_navigation(gravity_field)
        ImGui.tree_pop
      end

      if ImGui.tree_node_ex("Pilot", ImGui::ImGuiTreeNodeFlags.new(ImGui::ImGuiTreeNodeFlags::DefaultOpen))
        draw_pilot
        ImGui.tree_pop
      end

      ImGui.end
    end
  end

  def draw_bodies
    if ImGui.begin("Bodies")
      if ImGui.tree_node_ex("Bodies", ImGui::ImGuiTreeNodeFlags.new(ImGui::ImGuiTreeNodeFlags::DefaultOpen))

        draw_table(title: "Bodies", headers: {"Name", "X", "Y"}) do
          @projectables.each do |projectable|
            draw_table_line projectable.name, projectable.position[0].to_s, projectable.position[1].to_s
          end
          if !(click_coordinates = @click_coordinates).nil?
            draw_table_line "[click]", click_coordinates[0].to_s, click_coordinates[1].to_s
          end
        end

        ImGui.tree_pop
      end
      ImGui.end
    end
  end

end
