module PhysicsSandboxGrid

  MAIN_LINES_REAL_SPACE = 1000
  SECONDARY_LINES_AMOUNT = 3
  SECONDARY_LINES_SPACE = MAIN_LINES_REAL_SPACE / SECONDARY_LINES_AMOUNT
  SECONDARY_LINES_COLOR = SF::Color.new(255, 255, 255, 30)

  def absolute_coordinates(pos, offset)
    SF.vector2f(pos[0] + offset[0], pos[1] + offset[1])
  end

  # this one will be dynamic to evaluate distances
  def draw_grid
    main_lines = Array(SF::VertexBuffer).new
    secondary_lines = Array(SF::VertexBuffer).new
    width = @width
    height = @height
    max_size = {width, height}.max
    offset = {
      -@center_position[0] + (@center_position[0] // MAIN_LINES_REAL_SPACE) * MAIN_LINES_REAL_SPACE,
      -@center_position[1] + (@center_position[1] // MAIN_LINES_REAL_SPACE) * MAIN_LINES_REAL_SPACE,
    }

    main_lines_amount = (max_size / MAIN_LINES_REAL_SPACE).ceil.to_i32 + 1
    start = -main_lines_amount // 2
    stop = start + main_lines_amount
    pp({
         center_position: @center_position,
         offset: offset,
         max_size: max_size,
         start: start,
         stop: stop,
         main_lines_amount: main_lines_amount,
       })

    (start..stop).each do |index|
      main_line_v = SF::VertexBuffer.new(SF::LineStrip)
      main_line_h = SF::VertexBuffer.new(SF::LineStrip)
      main_lines << main_line_v
      main_lines << main_line_h
      main_line_v.create(2)
      main_line_h.create(2)
      x_start = MAIN_LINES_REAL_SPACE * index
      x_end = width
      y_start = MAIN_LINES_REAL_SPACE * index
      y_end = height
      # pp({
      #      center_position: @center_position,
      #      index: index,
      #      v_start: v_start,
      #      x_end: x_end,
      #      y_start: y_start,
      #      h_end: h_end,
      #      x_start: x_start,
      #      y_start: y_start,
      #    })
      main_line_v_buffer_0 = SF::Vertex.new(
        absolute_coordinates({x_start, start * MAIN_LINES_REAL_SPACE}, offset),
      )
      main_line_v_buffer_1 = SF::Vertex.new(
        absolute_coordinates({x_start, y_end}, offset),
      )
      main_line_h_buffer_0 = SF::Vertex.new(
        absolute_coordinates({start * MAIN_LINES_REAL_SPACE, y_start}, offset),
      )
      main_line_h_buffer_1 = SF::Vertex.new(
        absolute_coordinates({x_end, y_start}, offset),
      )
      main_line_v_buffer = Slice[main_line_v_buffer_0, main_line_v_buffer_1]
      main_line_h_buffer = Slice[main_line_h_buffer_0, main_line_h_buffer_1]
      main_line_v.update(main_line_v_buffer, 0)
      main_line_h.update(main_line_h_buffer, 0)

      SECONDARY_LINES_AMOUNT.times do |subindex|
        secondary_line_v = SF::VertexBuffer.new(SF::LineStrip)
        secondary_line_h = SF::VertexBuffer.new(SF::LineStrip)
        secondary_lines << secondary_line_v
        secondary_lines << secondary_line_h
        secondary_line_v.create(2)
        secondary_line_h.create(2)
        secondary_x_start = SECONDARY_LINES_SPACE * subindex
        secondary_y_start = SECONDARY_LINES_SPACE * subindex
        secondary_line_v_buffer_0 = SF::Vertex.new(
          absolute_coordinates({x_start + secondary_x_start, start * MAIN_LINES_REAL_SPACE}, offset),
          color: SECONDARY_LINES_COLOR,
        )
        secondary_line_v_buffer_1 = SF::Vertex.new(
          absolute_coordinates({x_start + secondary_x_start, y_end}, offset),
          color: SECONDARY_LINES_COLOR,
        )
        secondary_line_h_buffer_0 = SF::Vertex.new(
          absolute_coordinates({start * MAIN_LINES_REAL_SPACE, y_start + secondary_y_start}, offset),
          color: SECONDARY_LINES_COLOR,
        )
        secondary_line_h_buffer_1 = SF::Vertex.new(
          absolute_coordinates({x_end, y_start + secondary_y_start}, offset),
          color: SECONDARY_LINES_COLOR,
        )
        secondary_line_v_buffer = Slice[secondary_line_v_buffer_0, secondary_line_v_buffer_1]
        secondary_line_h_buffer = Slice[secondary_line_h_buffer_0, secondary_line_h_buffer_1]
        secondary_line_v.update(secondary_line_v_buffer, 0)
        secondary_line_h.update(secondary_line_h_buffer, 0)
      end

    end

    main_lines.each do |line|
      draw(line)
    end
    secondary_lines.each do |line|
      draw(line)
    end
  end

end
